@@ -2,6 +2,8 @@
 #include <stdlib.h>
 
 #include <glib.h>
+#include <glib/gi18n.h>
+
 #include <bickley/bkl-item.h>
 #include <bickley/bkl-item-audio.h>
 #include <bickley/bkl-item-image.h>
@@ -24,7 +26,7 @@ make_artist_name (GPtrArray *artists)
     char *str;
 
     if (artists == NULL) {
-        return g_strdup ("Unknown");
+        return g_strdup (_("Unknown Artist"));
     }
 
     builder = g_string_new (artists->pdata[0]);
@@ -34,7 +36,7 @@ make_artist_name (GPtrArray *artists)
     }
 
     if (artists->len > MAX_ARTISTS_IN_NAME) {
-        g_string_append_printf (builder, " and %d more",
+        g_string_append_printf (builder, _(" and %d more"),
                                 artists->len - MAX_ARTISTS_IN_NAME);
     }
 
@@ -275,7 +277,7 @@ add_audio_item (HrnClusterTree *tree,
     /* Each item can only be in one album */
     album_name = bkl_item_audio_get_album (audio);
     if (album_name == NULL) {
-        album_name = "Unknown";
+        album_name = _("Unknown");
         canonical_album = g_strdup ("Unknown");
     } else {
         make_album_name (album_name);
@@ -545,8 +547,10 @@ sort_months (gconstpointer a,
 }
 
 static char *months[12] = {
-    "January", "Feburary", "March", "April", "May", "June", "July",
-    "August", "September", "October", "November", "December"
+    N_("January"), N_("Feburary"), N_("March"),
+    N_("April"), N_("May"), N_("June"), N_("July"),
+    N_("August"), N_("September"), N_("October"),
+    N_("November"), N_("December")
 };
 
 static void
@@ -570,28 +574,23 @@ add_image_item (HrnClusterTree *tree,
     if (date_taken == NULL || *date_taken == '\0') {
         m = 13;
         y = 0;
-        month = g_strdup ("Unknown");
-        year = g_strdup ("Unknown");
+        month = g_strdup (_("Unknown"));
+        year = g_strdup (_("Unknown"));
     } else {
         m = atoi (date_taken + 5);
         y = atoi (date_taken);
 
         if (m == 0) {
-            month = g_strdup ("Unknown");
+            month = g_strdup (_("Unknown"));
         } else {
-            month = g_strdup (months[m - 1]);
+            month = g_strdup (_(months[m - 1]));
         }
 
         if (y == 0) {
-            year = g_strdup ("Unknown");
+            year = g_strdup (_("Unknown"));
         } else {
             year = g_strndup (date_taken, 4);
         }
-
-#if 0
-        g_print ("%s: %s %s (%s)\n", date_taken, month, year,
-                 bkl_item_get_uri (image));
-#endif
     }
 
     canonical = g_strdup_printf ("%s - %s", year, month);
@@ -621,6 +620,7 @@ add_image_item (HrnClusterTree *tree,
         thumb = bkl_item_extended_get_thumbnail ((BklItemExtended *) image);
         if (thumb) {
             year_cluster->thumbnail_uri = g_strdup (thumb);
+            year_cluster->orient = g_strdup (bkl_item_image_get_orientation (image));
         }
     }
 
@@ -651,6 +651,7 @@ add_image_item (HrnClusterTree *tree,
         thumb = bkl_item_extended_get_thumbnail ((BklItemExtended *) image);
         if (thumb) {
             month_cluster->thumbnail_uri = g_strdup (thumb);
+            month_cluster->orient = g_strdup (bkl_item_image_get_orientation (image));
         }
     }
 
@@ -939,6 +940,12 @@ remove_child_from_parent (HrnClusterNode *child,
     hrn_cluster_node_remove_child (parent, child);
     g_object_unref (child);
 
+    if (parent->type == HRN_CLUSTER_NODE_TYPE_AUDIO_ROOT ||
+        parent->type == HRN_CLUSTER_NODE_TYPE_IMAGE_ROOT ||
+        parent->type == HRN_CLUSTER_NODE_TYPE_VIDEO_ROOT) {
+        return;
+    }
+
     /* Check if the parent has any children left,
        it might need removed too */
     if (g_sequence_get_length (parent->children) == 0) {
