@@ -16,6 +16,12 @@
  * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+#ifndef __HRN_DRAG_N_DROP_H__
+#define __HRN_DRAG_N_DROP_H__
+
+#define HRN_DROP_MASK_QUEUE    (1<<0)
+#define HRN_DROP_MASK_SIDEBAR  (1<<1)
+
 void hrn_actor_make_draggable (ClutterActor *actor,
                                ClutterActor *drag_pad,
                                guint         mask,
@@ -23,3 +29,5 @@ void hrn_actor_make_draggable (ClutterActor *actor,
                                GCallback     can_drop_cb,
                                GCallback     drop_cb,
                                gpointer      data);
+
+#endif
