@@ -452,9 +452,9 @@ hrn_sidebar_item_init (HrnSidebarItem *self)
     NbtkWidget *title_table;
     int i;
     struct _DefaultEntry default_entries[] = {
-        { N_("Audio"), "icon-audio", G_CALLBACK (audio_item_clicked) },
-        { N_("Video"), "icon-video", G_CALLBACK (video_item_clicked) },
-        { N_("Image"), "icon-images", G_CALLBACK (image_item_clicked) },
+        { N_("Music"), "icon-audio", G_CALLBACK (audio_item_clicked) },
+        { N_("Pictures"), "icon-images", G_CALLBACK (image_item_clicked) },
+        { N_("Videos"), "icon-video", G_CALLBACK (video_item_clicked) },
         { NULL, NULL, NULL }
     };
 
