@@ -1,5 +1,6 @@
 #include "hrn.h"
 
+#include "hrn-cluster-tree.h"
 #include "hrn-source.h"
 
 enum
@@ -12,6 +13,8 @@ enum
   ITEM_CHANGED,
   ITEM_ADDED,
   ITEM_REMOVED,
+  STARTED,
+  FINISHED,
   LAST_SIGNAL
 };
 
@@ -20,13 +23,18 @@ struct _HrnSourcePrivate
   BklSourceClient *client;
   BklDB           *db;
   GSequence       *index;
-  GPtrArray       *items;
-  GHashTable      *uri_to_item;
+
+  HrnClusterTree *tree;
+  GHashTable     *uri_to_item;
+
+  gboolean is_indexing;
 
   guint32 changed_id;
   guint32 added_id;
   guint32 removed_id;
   guint32 index_changed_id;
+  guint32 started_id;
+  guint32 finished_id;
 };
 
 #define GET_PRIVATE(obj)    (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
@@ -35,34 +43,46 @@ struct _HrnSourcePrivate
 G_DEFINE_TYPE (HrnSource, hrn_source, G_TYPE_OBJECT);
 static guint32 signals[LAST_SIGNAL] = { 0, };
 
+#if 0
+static void
+free_items (GPtrArray *items)
+{
+  if (items == NULL) {
+    return;
+  }
+
+  if (items) {
+    int i;
+    for (i = 0; i < items->len; i++) {
+      g_object_unref (items->pdata[i]);
+    }
+
+    g_ptr_array_free (items, TRUE);
+  }
+}
+#endif
+
 static void
 hrn_source_finalize (GObject *object)
 {
   HrnSource        *self = (HrnSource *) object;
   HrnSourcePrivate *priv = self->priv;
-  int               i;
 
-  if (priv->uri_to_item)
-    {
-      g_hash_table_destroy (priv->uri_to_item);
-      priv->uri_to_item = NULL;
-    }
-
-  if (priv->items)
-    {
-      for (i = 0; i < priv->items->len; i++)
-        {
-          g_object_unref (priv->items->pdata[i]);
-        }
-      g_ptr_array_free (priv->items, TRUE);
-      priv->items = NULL;
-    }
-
-  if (priv->index)
-    {
+  if (priv->index) {
       g_sequence_free (priv->index);
       priv->index = NULL;
-    }
+  }
+
+  if (priv->tree) {
+      hrn_cluster_tree_free (priv->tree);
+      priv->tree = NULL;
+  }
+
+  if (priv->uri_to_item) {
+      g_hash_table_destroy (priv->uri_to_item);
+      priv->uri_to_item = NULL;
+  }
+
   G_OBJECT_CLASS (hrn_source_parent_class)->finalize (object);
 }
 
@@ -133,6 +153,18 @@ hrn_source_class_init (HrnSourceClass *klass)
                                         G_SIGNAL_NO_RECURSE, 0, NULL, NULL,
                                         g_cclosure_marshal_VOID__OBJECT,
                                         G_TYPE_NONE, 1, BKL_TYPE_ITEM);
+  signals[STARTED] = g_signal_new ("index-started",
+                                   G_TYPE_FROM_CLASS (klass),
+                                   G_SIGNAL_RUN_FIRST |
+                                   G_SIGNAL_NO_RECURSE, 0, NULL, NULL,
+                                   g_cclosure_marshal_VOID__VOID,
+                                   G_TYPE_NONE, 0);
+  signals[FINISHED] = g_signal_new ("index-finished",
+                                    G_TYPE_FROM_CLASS (klass),
+                                    G_SIGNAL_RUN_FIRST |
+                                    G_SIGNAL_NO_RECURSE, 0, NULL, NULL,
+                                    g_cclosure_marshal_VOID__VOID,
+                                    G_TYPE_NONE, 0);
 }
 
 static void
@@ -143,7 +175,8 @@ hrn_source_init (HrnSource *self)
   self->priv = GET_PRIVATE (self);
   priv       = self->priv;
 
-  priv->uri_to_item = g_hash_table_new (g_str_hash, g_str_equal);
+  priv->uri_to_item = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                             NULL, g_object_unref);
 }
 
 static void
@@ -160,6 +193,17 @@ uri_changed_cb (BklSourceClient *client, const char      *uri,
   g_signal_emit (source, signals[ITEM_CHANGED], 0, item);
 }
 
+static void
+insert_item (HrnSource *source,
+             BklItem   *item)
+{
+  HrnSourcePrivate *priv = source->priv;
+
+  hrn_cluster_tree_add_item (priv->tree, item);
+  g_hash_table_insert (priv->uri_to_item,
+                       (char *) bkl_item_get_uri (item), item);
+}
+
 static void
 uri_added_cb (BklSourceClient *client, const char      *uri,
               HrnSource       *source)
@@ -176,30 +220,33 @@ uri_added_cb (BklSourceClient *client, const char      *uri,
       return;
     }
 
-  g_ptr_array_add (priv->items, item);
-  g_hash_table_insert (priv->uri_to_item,
-                       (char *) bkl_item_get_uri (item), item);
+  insert_item (source, item);
+
+  /* FIXME: Do we need this emitted now,
+     as the signals are really emitted
+     on the tree */
   g_signal_emit (source, signals[ITEM_ADDED], 0, item);
 }
 
 static void
-uri_removed_cb (BklSourceClient *client, const char      *uri,
+uri_removed_cb (BklSourceClient *client,
+                const char      *uri,
                 HrnSource       *source)
 {
   HrnSourcePrivate *priv = source->priv;
   BklItem          *item;
 
   item = g_hash_table_lookup (priv->uri_to_item, uri);
-  if (item == NULL)
-    {
+  if (item == NULL) {
       return;
-    }
+  }
 
-  g_hash_table_remove (priv->uri_to_item, uri);
-  g_ptr_array_remove (priv->items, item);
+  hrn_cluster_tree_remove_item (priv->tree, item);
+  /* FIXME: Do we need this signal emitted?
+     Emit the signal before removing the item from the hash table
+     as the hash table will unref the item */
   g_signal_emit (source, signals[ITEM_REMOVED], 0, item);
-
-  g_object_unref (item);
+  g_hash_table_remove (priv->uri_to_item, uri);
 }
 
 static int
@@ -242,14 +289,28 @@ index_changed_cb (BklSourceClient *client,
     /* FIXME: Should update the search here */
 }
 
+static void
+index_started_cb (BklSourceClient *client,
+                  HrnSource       *source)
+{
+  g_signal_emit (source, signals[STARTED], 0);
+}
+
+static void
+index_finished_cb (BklSourceClient *client,
+                   HrnSource       *source)
+{
+  g_signal_emit (source, signals[FINISHED], 0);
+}
 
 HrnSource *
 hrn_source_new (BklSourceClient *client)
 {
   HrnSource        *source;
   HrnSourcePrivate *priv;
+  GPtrArray        *items;
   GError           *error = NULL;
-  int               i;
+  int i;
 
   source = g_object_new (HRN_TYPE_SOURCE, NULL);
   priv   = source->priv;
@@ -263,6 +324,10 @@ hrn_source_new (BklSourceClient *client)
                                        G_CALLBACK (uri_removed_cb), source);
   priv->index_changed_id = g_signal_connect (client, "index-changed",
                                        G_CALLBACK (index_changed_cb), source);
+  priv->started_id = g_signal_connect (client, "in-progress",
+                                       G_CALLBACK (index_started_cb), source);
+  priv->finished_id = g_signal_connect (client, "complete",
+                                        G_CALLBACK (index_finished_cb), source);
 
   priv->db = bkl_source_client_get_db (client);
   if (priv->db == NULL)
@@ -275,8 +340,8 @@ hrn_source_new (BklSourceClient *client)
 
   priv->index = bkl_db_get_index_words (priv->db);
 
-  priv->items = bkl_db_get_items (priv->db, FALSE, &error);
-  if (priv->items == NULL)
+  items = bkl_db_get_items (priv->db, FALSE, &error);
+  if (items == NULL)
     {
       g_warning ("Error getting items: %s", error->message);
       g_error_free (error);
@@ -285,14 +350,18 @@ hrn_source_new (BklSourceClient *client)
       return NULL;
     }
 
-  for (i = 0; i < priv->items->len; i++)
-    {
-      BklItem *item = priv->items->pdata[i];
+  priv->tree = hrn_cluster_tree_new ();
+  for (i = 0; i < items->len; i++) {
+      BklItem *item = items->pdata[i];
+
       g_hash_table_insert (priv->uri_to_item,
                            (char *) bkl_item_get_uri (item), item);
-    }
+      hrn_cluster_tree_add_item (priv->tree, item);
+  }
+
+  /* hrn_cluster_tree_dump_items (priv->tree); */
+  g_ptr_array_free (items, TRUE);
 
-  /* FIXME: Deal with the index */
   return source;
 }
 
@@ -329,11 +398,36 @@ hrn_source_get_name (HrnSource *source)
 }
 
 GPtrArray *
-hrn_source_get_items (HrnSource *source)
+hrn_source_get_audio_items (HrnSource *source)
 {
+#if 0
   HrnSourcePrivate *priv = source->priv;
 
-  return priv->items;
+  return priv->audio_items;
+#endif
+  return NULL;
+}
+
+GPtrArray *
+hrn_source_get_image_items (HrnSource *source)
+{
+#if 0
+  HrnSourcePrivate *priv = source->priv;
+
+  return priv->image_items;
+#endif
+  return NULL;
+}
+
+GPtrArray *
+hrn_source_get_video_items (HrnSource *source)
+{
+#if 0
+  HrnSourcePrivate *priv = source->priv;
+
+  return priv->video_items;
+#endif
+  return NULL;
 }
 
 BklItem *
@@ -348,8 +442,8 @@ hrn_source_get_item (HrnSource *source, const char *uri)
 
 struct _Bartowski
 {
-  int    word_count;
-  GList *chuck;
+    int    word_count;
+    GHashTable *chuck;
 };
 
 static void
@@ -360,14 +454,15 @@ flash_intersect (gpointer key, gpointer value, gpointer userdata)
 
   /* We generate the intersection of the sets by including only
      uris that have a word count equal to the number of sets */
-  if (count == bart->word_count)
-    {
-      bart->chuck = g_list_prepend (bart->chuck, g_strdup (key));
-    }
+  if (count == bart->word_count) {
+      char *data = g_strdup (key);
+
+      g_hash_table_insert (bart->chuck, data, data);
+  }
 }
 
 
-static GList *
+static GHashTable *
 find_intersect (GPtrArray *sets)
 {
   struct _Bartowski *bart;
@@ -402,7 +497,8 @@ find_intersect (GPtrArray *sets)
 
   bart             = g_newa (struct _Bartowski, 1);
   bart->word_count = sets->len;
-  bart->chuck      = NULL;
+  bart->chuck      = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                            g_free, NULL);
 
   g_hash_table_foreach (set, flash_intersect, bart);
   g_hash_table_destroy (set);
@@ -418,14 +514,15 @@ search_index_for_words (HrnSource *source, const char *text)
   char     **search_terms;
   GPtrArray *words;
   int        k;
-  GSequence *index;
+  GSequence *idx;
 
 
   search_text  = g_ascii_strup (text, -1);
-  search_terms = g_strsplit (search_text, " ", -1);
+  /* FIXME: Should we just remove all punctuation? */
+  search_terms = g_strsplit_set (search_text, " :➜", 0);
   g_free (search_text);
 
-  index        = hrn_source_get_index (source);
+  idx = hrn_source_get_index (source);
 
   /* Search each index for possible substrings in search_words */
   words = g_ptr_array_new ();
@@ -445,7 +542,7 @@ search_index_for_words (HrnSource *source, const char *text)
 
       g_ptr_array_add (words, possible);
 
-      iter = g_sequence_get_begin_iter (index);
+      iter = g_sequence_get_begin_iter (idx);
 
       while (!g_sequence_iter_is_end (iter))
         {
@@ -542,51 +639,23 @@ search_for_uris (HrnSource *source, GPtrArray *possible_words)
       g_ptr_array_add (possible_uris, results);
     }
 
-
   return possible_uris;
 }
 
-
-static GPtrArray *
-copy_and_ref_array (GPtrArray *original)
+void
+hrn_source_filter (HrnSource   *source,
+                   const gchar *text)
 {
-  GPtrArray *clone;
-  int        i;
-
-  if (original == NULL)
-    {
-      return NULL;
-    }
-
-  clone = g_ptr_array_sized_new (original->len);
-  for (i = 0; i < original->len; i++)
-    {
-      if (original->pdata[i] && G_IS_OBJECT (original->pdata[i]))
-        g_ptr_array_add (clone, g_object_ref (original->pdata[i]));
-      else {
-        g_warning ("%s: invalid object expected a HRN_ITEM, is it unreferenced?\n", G_STRLOC);
-        g_warning ("Invalid object is %p", original->pdata[i]);
-      }
-    }
-
-  return clone;
-}
-
-GPtrArray *
-hrn_source_query (HrnSource *source, const gchar *text)
-{
-  GPtrArray *results;
-  /*Source *source = priv->dbs->pdata[i];*/
-  GList     *r = NULL;
+  HrnSourcePrivate *priv = source->priv;
+  GHashTable *results;
   GPtrArray *index_words;
   GPtrArray *possible_results;
   int i;
 
-  if (!text || strlen (text) == 0)
-    {
-      return copy_and_ref_array (hrn_source_get_items (source));
-    }
-  results = g_ptr_array_new ();
+  if (text == NULL || *text == '\0') {
+      hrn_cluster_tree_filter_items (priv->tree, NULL);
+      return;
+  }
 
   /* @index_words is a GPtrArray containing GPtrArrays
      that contain all the possible words for each word in @text */
@@ -606,27 +675,7 @@ hrn_source_query (HrnSource *source, const gchar *text)
   }
   g_ptr_array_free (index_words, TRUE);
 
-  {
-    r = find_intersect (possible_results);
-    GList *iter;
-    for (iter = r; iter; iter = iter->next)
-      {
-        BklItem *item;
-        item = hrn_source_get_item (source, iter->data);
-        if (item && G_IS_OBJECT (item))
-          {
-            g_object_ref (item);
-            g_ptr_array_add (results, item);
-          }
-        else
-          {
-            g_warning ("%s: %s got a NULL BklItem from source\n", G_STRLOC,
-                       (gchar*) iter->data);
-          }
-        g_free (iter->data);
-      }
-    g_list_free (r);
-  }
+  results = find_intersect (possible_results);
 
   for (i = 0; i < possible_results->len; i++) {
       GList *res = possible_results->pdata[i];
@@ -638,6 +687,17 @@ hrn_source_query (HrnSource *source, const gchar *text)
       g_list_free (res);
   }
   g_ptr_array_free (possible_results, TRUE);
-  return results;
+
+  hrn_cluster_tree_filter_items (priv->tree, results);
+  g_hash_table_destroy (results);
+
+  hrn_cluster_tree_filter_nodes (priv->tree, text);
 }
 
+HrnClusterTree *
+hrn_source_get_cluster_tree (HrnSource *source)
+{
+    HrnSourcePrivate *priv = source->priv;
+
+    return priv->tree;
+}
